@@ -8,6 +8,7 @@ import numpy as np
 import pytest
 
 from ...grids import UnitGrid
+from ...tools.misc import skipUnlessModule
 from .. import FieldCollection, ScalarField, Tensor2Field, VectorField
 from ..base import FieldBase
 from .test_generic import iter_grids
@@ -73,6 +74,8 @@ def test_collections():
     with pytest.raises(KeyError):
         fields["42"] = 0
 
+    fields.plot()
+
 
 def test_collections_copy():
     """ test copying data of collections """
@@ -161,3 +164,53 @@ def test_from_scalar_expressions():
     assert fc[1].grid is grid
     np.testing.assert_allclose(fc[0].data, (np.arange(3) + 0.5) ** 2)
     np.testing.assert_allclose(fc[1].data, 1)
+
+
+@skipUnlessModule("napari")
+def test_interactive_collection_plotting():
+    """ test the interactive plotting """
+    grid = UnitGrid([3, 3])
+    sf = ScalarField.random_uniform(grid, 0.1, 0.9)
+    vf = VectorField.random_uniform(grid, 0.1, 0.9)
+    field = FieldCollection([sf, vf])
+    field.plot_interactive(viewer_args={"show": False, "close": True})
+
+
+def test_field_labels():
+    """ test the FieldCollection.labels property """
+    grid = UnitGrid([5])
+    s1 = ScalarField(grid, label="s1")
+    s2 = ScalarField(grid)
+    fc = FieldCollection([s1, s2])
+
+    assert fc.labels == ["s1", None]
+    assert len(fc.labels) == 2
+    assert fc.labels[0] == "s1"
+    fc.labels = ["a", "b"]
+    assert fc.labels == ["a", "b"]
+    fc.labels[0] = "c"
+    assert fc.labels == ["c", "b"]
+    assert str(fc.labels) == str(["c", "b"])
+    assert repr(fc.labels) == repr(["c", "b"])
+
+    assert fc.labels[0:1] == ["c"]
+    assert fc.labels[:] == ["c", "b"]
+    fc.labels[0:1] = "d"
+    assert fc.labels == ["d", "b"]
+
+    fc.labels[:] = "a"
+    assert fc.labels == ["a", "a"]
+
+    labels = fc.labels[:]
+    labels[0] = "e"
+    assert fc.labels == ["a", "a"]
+
+
+def test_collection_1_field():
+    """ test field collections with only one field """
+    grid = UnitGrid([3])
+    s1 = ScalarField(grid, label="a")
+    fc = FieldCollection([s1])
+    assert fc.labels == ["a"]
+
+    fc.plot()
