@@ -19,9 +19,9 @@ representations using :mod:`sympy`.
 import builtins
 import copy
 import logging
+import numbers
 import re
 from abc import ABCMeta, abstractproperty
-from numbers import Number
 from typing import Optional  # @UnusedImport
 from typing import Set  # @UnusedImport
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Sequence, Tuple, Union
@@ -32,6 +32,7 @@ import sympy
 from sympy.printing.pycode import PythonCodePrinter
 from sympy.utilities.lambdify import _get_namespace
 
+from ..tools.misc import Number, number, number_array
 from .cache import cached_method, cached_property
 from .docstrings import fill_in_docstring
 from .numba import convert_scalar, jit
@@ -42,21 +43,21 @@ if TYPE_CHECKING:
 
 @fill_in_docstring
 def parse_number(
-    expression: Union[str, float], variables: Dict[str, float] = None
-) -> float:
+    expression: Union[str, Number], variables: Dict[str, Number] = None
+) -> Number:
     r"""return a number compiled from an expression
 
     Warning:
         {WARNING_EXEC}
 
     Args:
-        expression (str or float):
+        expression (str or Number):
             An expression that can be interpreted as a number
         variables (dict):
             A dictionary of values that replace variables in the expression
 
     Returns:
-        float: the calculated value
+        Number: the calculated value
     """
     from sympy.parsing import sympy_parser
 
@@ -65,12 +66,13 @@ def parse_number(
 
     expr = sympy_parser.parse_expr(str(expression))
     try:
-        value = float(expr.subs(variables))
+        value = number(expr.subs(variables))
     except TypeError as err:
         if not err.args:
             err.args = ("",)
         err.args = err.args + (f"Expression: `{expr}`",)
         raise
+
     return value
 
 
@@ -150,6 +152,11 @@ class ExpressionBase(metaclass=ABCMeta):
         """ bool: whether the expression is a constant """
         return len(self._sympy_expr.free_symbols) == 0
 
+    @property
+    def complex(self) -> bool:
+        """ bool: whether the expression contains the imaginary unit I """
+        return sympy.I in self._sympy_expr.atoms()
+
     @abstractproperty
     def shape(self) -> Tuple[int, ...]:
         pass
@@ -232,7 +239,10 @@ class ExpressionBase(metaclass=ABCMeta):
             )
 
     def _get_function(
-        self, single_arg: bool = False, user_funcs: Dict[str, Callable] = None
+        self,
+        single_arg: bool = False,
+        user_funcs: Dict[str, Callable] = None,
+        prepare_compilation: bool = False,
     ) -> Callable:
         """return function evaluating expression
 
@@ -243,6 +253,9 @@ class ExpressionBase(metaclass=ABCMeta):
                 to be supplied separately
             user_funcs (dict):
                 Additional functions that can be used in the expression
+            prepare_compilation (bool):
+                Determines whether all user functions are marked with
+                :func:`numba.extending.register_jitable` to prepare for compilation.
 
         Returns:
             function: the function
@@ -250,6 +263,8 @@ class ExpressionBase(metaclass=ABCMeta):
         user_functions = self.user_funcs.copy()
         if user_funcs is not None:
             user_functions.update(user_funcs)
+        if prepare_compilation:
+            user_functions = {k: jit(v) for k, v in user_functions.items()}
         user_dict = {k: k for k in user_functions}
 
         printer = NumpyArrayPrinter(
@@ -270,7 +285,9 @@ class ExpressionBase(metaclass=ABCMeta):
         )
 
     @cached_method()
-    def _get_function_cached(self, single_arg: bool = False) -> Callable:
+    def _get_function_cached(
+        self, single_arg: bool = False, prepare_compilation: bool = False
+    ) -> Callable:
         """return function evaluating expression
 
         Args:
@@ -278,11 +295,14 @@ class ExpressionBase(metaclass=ABCMeta):
                 Determines whether the returned function accepts all variables
                 in a single argument as an array or whether all variables need
                 to be supplied separately
+            prepare_compilation (bool):
+                Determines whether all user functions are marked with
+                :func:`numba.extending.register_jitable` to prepare for compilation.
 
         Returns:
             function: the function
         """
-        return self._get_function(single_arg)
+        return self._get_function(single_arg, prepare_compilation=prepare_compilation)
 
     def __call__(self, *args, **kwargs):
         """ return the value of the expression for the given values """
@@ -300,7 +320,10 @@ class ExpressionBase(metaclass=ABCMeta):
         Returns:
             function: the compiled function
         """
-        func = self._get_function_cached(single_arg=single_arg)
+        # compile the actual expression
+        func = self._get_function_cached(
+            single_arg=single_arg, prepare_compilation=True
+        )
         return jit(func)  # type: ignore
 
 
@@ -362,7 +385,7 @@ class ScalarExpression(ExpressionBase):
                 "Expression must be provided as string and not as a callable function"
             )
 
-        elif isinstance(expression, Number):
+        elif isinstance(expression, numbers.Number):
             # expression is a simple number
             sympy_expr = sympy.Float(expression)
 
@@ -384,10 +407,10 @@ class ScalarExpression(ExpressionBase):
         )
 
     @property
-    def value(self) -> float:
+    def value(self) -> Number:
         """ float: the value for a constant expression """
         if self.constant:
-            return float(self._sympy_expr)
+            return number(self._sympy_expr)
         else:
             raise TypeError("Only constant expressions have a defined value")
 
@@ -552,7 +575,7 @@ class TensorExpression(ExpressionBase):
     def value(self):
         """ the value for a constant expression """
         if self.constant:
-            return np.array(self._sympy_expr.tolist(), dtype=np.double)
+            return number_array(self._sympy_expr.tolist())
         else:
             raise TypeError("Only constant expressions have a defined value")
 
